#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#if defined(_MSC_VER) && defined(_DEBUG)
/* MSVC debug hook to prevent dialogs when running from DART.  */
# include <crtdbg.h>
static int TestDriverDebugReport(int type, char* message, int* retVal)
{
  (void)type; (void)retVal;
  fprintf(stderr, message);
  exit(1);
  return 0;
}
#endif

/* Forward declare test functions. */

int ImportExport(int, char*[]);

/* Create map.  */

typedef int (*MainFuncPointer)(int , char*[]);
typedef struct
{
  const char* name;
  MainFuncPointer func;
} functionMapEntry;

functionMapEntry cmakeGeneratedFunctionMapEntries[] = {
  {
    "ImportExport",
    ImportExport
  },
};

/* Allocate and create a lowercased copy of string
   (note that it has to be free'd manually) */

char* lowercase(const char *string)
{
  char *new_string, *p;
  new_string = (char *)malloc(sizeof(char) * (size_t)(strlen(string) + 1));
  if (!new_string)
    {
    return NULL;
    }
  strcpy(new_string, string);
  p = new_string;
  while (*p != 0)
    {
    *p = (char)tolower(*p);
    ++p;
    }
  return new_string;
}

int main(int ac, char *av[])
{
  int i, NumTests, testNum, partial_match;
  char *arg, *test_name;
  
#if defined(_MSC_VER) && defined(_DEBUG)
  /* If running from DART, put in debug hook.  */
  if(getenv("DART_TEST_FROM_DART"))
    {
    _CrtSetReportHook(TestDriverDebugReport);
    }
#endif
  
  NumTests = 1;
  
  /* If no test name was given */
  if (ac < 2)
    {
    /* Ask for a test.  */
    printf("Available tests:\n");
    for (i =0; i < NumTests; ++i)
      {
      printf("%3d. %s\n", i, cmakeGeneratedFunctionMapEntries[i].name);
      }
    printf("To run a test, enter the test number: ");
    testNum = 0;
    scanf("%d", &testNum);
    if (testNum >= NumTests)
      {
      printf("%3d is an invalid test number.\n", testNum);
      return -1;
      }
    return (*cmakeGeneratedFunctionMapEntries[testNum].func)(ac-1, av+1);
    }
  
  /* If partial match is requested.  */
  partial_match = (strcmp(av[1], "-R") == 0) ? 1 : 0;
  if (partial_match && ac < 3)
    {
    printf("-R needs an additional parameter.\n");
    return -1;
    }
  
  arg = lowercase(av[1 + partial_match]);
  for (i =0; i < NumTests; ++i)
    {
    test_name = lowercase(cmakeGeneratedFunctionMapEntries[i].name);
    if (partial_match && strstr(test_name, arg) != NULL)
      {
      free(arg);
      free(test_name);
      return (*cmakeGeneratedFunctionMapEntries[i].func)(ac - 2, av + 2);
      }
    else if (!partial_match && strcmp(test_name, arg) == 0)
      {
      free(arg);
      free(test_name);
      return (*cmakeGeneratedFunctionMapEntries[i].func)(ac - 1, av + 1);
      }
    free(test_name);
    }
  free(arg);
  
  /* Nothing was run, display the test names.  */
  printf("Available tests:\n");
  for (i =0; i < NumTests; ++i)
    {
    printf("%3d. %s\n", i, cmakeGeneratedFunctionMapEntries[i].name);
    }
  printf("Failed: %s is an invalid test name.\n", av[1]);
  
  return -1;
}
